import logging

from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from collective.easyform.api import get_actions, set_actions
from collective.lineage.utils import enable_childsite
from plone import api
from plone.app.contenttypes.browser.collection import CollectionView
from plone.app.dexterity.behaviors.constrains import ENABLED
from plone.app.dexterity.behaviors.exclfromnav import IExcludeFromNavigation
from plone.app.event.dx.behaviors import IEventContact
from plone.event.interfaces import IEventAccessor
from plone.event.interfaces import IOccurrence
from plone.supermodel import model
from plone.app.textfield.value import RichTextValue
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import \
    WorkflowPolicyConfig_id

from .vocabularies import PRESENTATION_DURATION_TYPES, LEVEL_TYPES, \
    AUDIENCE_TYPES

REGISTRATION_FORM_ID = 'registration'
REGISTRATION_FORM_ENABLED_ATTR = 'enable_standard_registration_form_'

DOCUMENTS_FORM_ID = 'submit-documents'
DOCUMENTS_FORM_ENABLED_ATTR = 'enable_document_submission_form_'

REGISTRATION_INTERNATIONAL_FORM_ID = 'registration-international'
REGISTRATION_INTERNATIONAL_FORM_ENABLED_ATTR = 'enable_international_registration_form_'

REGISTRATION_US_CITIZEN_FORM_ID = 'registration-us-citizen'
REGISTRATION_US_CITIZEN_FORM_ENABLED_ATTR = 'enable_us_citizen_registration_form_'

SET_CONFERENCE_WORKFLOW_ATTR = 'set_conference_to_private_'


class IConference(model.Schema):
    model.load('models/conference.xml')


class ConferenceView(CollectionView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.data = IEventAccessor(context)

    def __call__(self):
        if IOccurrence.providedBy(self.context):
            # The transient Occurrence objects cannot be edited. disable the
            # edit border for them.
            self.request.set('disable_border', True)
        return self.index()  # render me.

    def duration_vocab(self):
        return PRESENTATION_DURATION_TYPES

    def level_vocab(self):
        return LEVEL_TYPES

    def audience_vocab(self):
        return AUDIENCE_TYPES

    def speakers(self):
        """ parse speaker set into something template can iterate over
        """
        speaker_set = self.context.speaker
        speakers = [api.content.get(UID=x) for x in speaker_set]
        speaker_data = []
        for x in speakers:
            scale_func = api.content.get_view(name='images', context=x, request=self.request)
            # scale choices are taken from portal_registry/edit/plone.allowed_sizes
            scaled_image = getattr(x.aq_explicit, 'headshot', False) and scale_func.scale('headshot', width=150, height=150)
            if scaled_image:
                tag = scaled_image.tag(css_class='image-left')
            else:
                tag = ''
            speaker_data.append({'url': x.absolute_url(), 'name': x.title, 'headshot': tag})
        return speaker_data

    def vocab_title(self, values, vocab):
        """ return title when given the vocabulary and the value key
        """
        if not isinstance(values, (list, set)):
            result = vocab.getTerm(values).title
            if vocab == AUDIENCE_TYPES:
                return [result] # hacky fix
            return result
        return [vocab.getTerm(x).title for x in values ]


def _set_mailer_adapter_recipient_email(conference, form):
    # set mailer adapter email address to be that of conference owner
    if form:
        actions = get_actions(form)
        mailer = actions['mailer']
        event_contact_aspect = IEventContact(conference)
        conference_contact_email = event_contact_aspect.contact_email
        if mailer.recipient_email != conference_contact_email:
            mailer.recipient_email = conference_contact_email
            set_actions(form, actions)

def create_conference_content(conference, event):
    logging.getLogger(__name__).info("setting up the new conference item %s" % conference.getId())

    # make the conference a lineage subsite
    enable_childsite(conference)

    # create conference-specific groups and roles
    conf_organizers_group_name = f"{conference.id}-organizers"
    conf_attendees_group_name = f"{conference.id}-attendees"
    conf_presenters_group_name = f"{conference.id}-presenters"
    api.group.create(groupname=conf_organizers_group_name)
    api.group.create(groupname=conf_attendees_group_name)
    api.group.create(groupname=conf_presenters_group_name)

    # check if private conference, set workflow policy
    _set_conference_workflow_policy(conference)

    # add current user to organizers group
    current_user = api.user.get_current()
    api.group.add_user(groupname=conf_organizers_group_name, username=current_user.getUserName())

    # grant local roles to groups in this conference
    api.group.grant_roles(
        groupname=conf_organizers_group_name, roles=['Owner'], obj=conference)


    # set default collection constraints
    conference.query = [
        {
            'i': 'portal_type',
            'o': 'plone.app.querystring.operation.selection.any',
            'v': ['presentation', 'training_class']
        }
    ]
    conference.sort_on = 'start'
    conference.sort_reversed = False

    # constrain types for conference object
    _constrain_types(conference, ['Folder', 'Document'])

    _create_conference_folder(conference, title='Speakers', allowed_types = ['person', 'keynoter'])
    _create_conference_folder(conference, title='Talks', allowed_types = ['presentation',])
    _create_conference_folder(conference, title='Classes', allowed_types = ['training_class',])
    documents = _create_conference_folder(conference, title='Documents', allowed_types = ['File',])

    # grant Reader to attendees group on the Documents folder
    documents.manage_setLocalRoles(conf_attendees_group_name, ['Reader',])

    # grant organizers group the Editor local role on the conference so members can edit and submit it
    conference.manage_setLocalRoles(conf_organizers_group_name, ['Editor','Contributor','Reader'])

    # copy template forms from site root
    if getattr(conference, REGISTRATION_FORM_ENABLED_ATTR, False):
        _copy_template_form(conference, "/" + REGISTRATION_FORM_ID)
    if getattr(conference, DOCUMENTS_FORM_ENABLED_ATTR, False):
        _copy_template_form(conference, "/" + DOCUMENTS_FORM_ID)
    if getattr(conference, REGISTRATION_INTERNATIONAL_FORM_ENABLED_ATTR, False):
        _copy_template_form(conference, "/" + REGISTRATION_INTERNATIONAL_FORM_ID)
    if getattr(conference, REGISTRATION_US_CITIZEN_FORM_ENABLED_ATTR, False):
        _copy_template_form(conference, "/" + REGISTRATION_US_CITIZEN_FORM_ID)


def _create_conference_folder(container, title, allowed_types):
    new_folder = api.content.create(
        type='Folder',
        title=title,
        container=container)
    _constrain_types(new_folder, allowed_types)
    return new_folder


def _constrain_types(folder, allowed_types):
    aspect = ISelectableConstrainTypes(folder)
    aspect.setConstrainTypesMode(ENABLED)
    aspect.setLocallyAllowedTypes(allowed_types)
    aspect.setImmediatelyAddableTypes(allowed_types)


def _set_conference_workflow_policy(conference):
    logging.getLogger(__name__).info("update conference workflow %s" % conference.getId())
    conf_organizers_group_name = f"{conference.id}-organizers"
    conf_attendees_group_name = f"{conference.id}-attendees"
    conf_presenters_group_name = f"{conference.id}-presenters"
    if getattr(conference, SET_CONFERENCE_WORKFLOW_ATTR, False):
        if getattr(conference, WorkflowPolicyConfig_id, False) is False:
            conference.manage_addProduct['CMFPlacefulWorkflow']\
                .manage_addWorkflowPolicyConfig()
        conf_config = getattr(conference, WorkflowPolicyConfig_id)
        conf_config.setPolicyIn('private-conference', True)
        conf_config.setPolicyBelow('private-conference', True)

        # grant Reviewer to attendees and presenters groups on the Conference folder
        conference.manage_setLocalRoles(conf_attendees_group_name, ['Reviewer',])
        conference.manage_setLocalRoles(conf_presenters_group_name, ['Reviewer',])
    else:
        if getattr(conference, WorkflowPolicyConfig_id, False):
            conf_config = getattr(conference, WorkflowPolicyConfig_id)
            conf_config.setPolicyIn('simple_publication_workflow', True)
            conf_config.setPolicyBelow('simple_publication_workflow', True)

            # remove Reviewer to attendees and presenters groups on the Conference folder
            api.group.revoke_roles(groupname=conf_attendees_group_name,
                                   roles=['Reviewer'],
                                   obj=conference)
            api.group.revoke_roles(groupname=conf_presenters_group_name,
                                   roles=['Reviewer'],
                                   obj=conference)


def _copy_template_form(conference, path):
    template_form = api.content.get(path=path)
    new_form = api.content.copy(source=template_form, target=conference)
    # make sure it's visible
    behavior_data = IExcludeFromNavigation(new_form)
    if behavior_data:
        behavior_data.exclude_from_nav = False
    # set mailer adapter email address to be that of conference owner
    _set_mailer_adapter_recipient_email(conference, new_form)


def update_conference_content(conference, event):
    logging.getLogger(__name__).info("update modified conference %s" % conference.getId())
    _set_mailer_adapter_recipient_email(conference, getattr(conference, REGISTRATION_FORM_ID, None))
    _set_mailer_adapter_recipient_email(conference, getattr(conference, DOCUMENTS_FORM_ID, None))
    _set_mailer_adapter_recipient_email(conference, getattr(conference, REGISTRATION_INTERNATIONAL_FORM_ID, None))
    _set_mailer_adapter_recipient_email(conference, getattr(conference, REGISTRATION_US_CITIZEN_FORM_ID, None))

    # check if private conference, set workflow policy
    _set_conference_workflow_policy(conference)


# Correct Rich Text output from form Generic Setup import
def _update_form_prologue_epilogue(context, form_id):
    portal = api.portal.get()
    form = getattr(portal, form_id, None)
    if form.formPrologue:
        prologue_text = form.formPrologue.raw
        form.formPrologue = RichTextValue(
            raw=prologue_text,
            mimeType='text/html',
            outputMimeType='text/html')
    if form.formEpilogue:
        epilogue_text = form.formEpilogue.raw
        form.formEpilogue = RichTextValue(
            raw=epilogue_text,
            mimeType='text/html',
            outputMimeType='text/html')


def update_forms_richtext(context):
    _update_form_prologue_epilogue(context, REGISTRATION_FORM_ID)
    _update_form_prologue_epilogue(context, DOCUMENTS_FORM_ID)
    _update_form_prologue_epilogue(context, REGISTRATION_INTERNATIONAL_FORM_ID)
    _update_form_prologue_epilogue(context, REGISTRATION_US_CITIZEN_FORM_ID)


def remove_conference_content(conference, event):
    logging.getLogger(__name__).info("remove groups for conference %s"
                                     % conference.getId())
    conf_groups = []
    group_types = ['-organizers', '-attendees', '-presenters']
    for group in group_types:
        conf_groups.append(f"{conference.id}{group}")

    for name in conf_groups:
        try:
            api.group.delete(name)
        except KeyError:
            logging.getLogger(__name__).info("unable to delete %s ,\
                                              group may not exist" % name)
